@@ -4,9 +4,16 @@
 import glob
 import optparse
 import os
+import platform
 import signal
 import sys
 
+# Make homebrew work by default
+if platform.system() == 'Darwin':
+    homebrew_mods = '/usr/local/lib/python'
+    if os.path.isdir(homebrew_mods):
+        sys.path.append(homebrew_mods)
+
 try:
     from PyQt4 import QtGui
     from PyQt4 import QtCore
@@ -31,10 +38,6 @@ from cola import resources
 from cola import signals
 from cola import utils
 from cola import version
-from cola.controllers.createtag import create_tag
-from cola.classic import cola_classic
-from cola.dag import git_dag
-from cola.stash import stash
 from cola.decorators import memoize
 from cola.main.view import MainView
 from cola.main.controller import MainController
@@ -62,6 +65,43 @@ def setup_environment():
     os.environ['PATH'] = path
     os.putenv('PATH', path)
 
+    # We don't ever want a pager
+    os.environ['GIT_PAGER'] = ''
+    os.putenv('GIT_PAGER', '')
+
+    # --- >8 --- >8 ---
+    # Git v1.7.10 Release Notes
+    # =========================
+    #
+    # Compatibility Notes
+    # -------------------
+    #
+    #  * From this release on, the "git merge" command in an interactive
+    #   session will start an editor when it automatically resolves the
+    #   merge for the user to explain the resulting commit, just like the
+    #   "git commit" command does when it wasn't given a commit message.
+    #
+    #   If you have a script that runs "git merge" and keeps its standard
+    #   input and output attached to the user's terminal, and if you do not
+    #   want the user to explain the resulting merge commits, you can
+    #   export GIT_MERGE_AUTOEDIT environment variable set to "no", like
+    #   this:
+    #
+    #        #!/bin/sh
+    #        GIT_MERGE_AUTOEDIT=no
+    #        export GIT_MERGE_AUTOEDIT
+    #
+    #   to disable this behavior (if you want your users to explain their
+    #   merge commits, you do not have to do anything).  Alternatively, you
+    #   can give the "--no-edit" option to individual invocations of the
+    #   "git merge" command if you know everybody who uses your script has
+    #   Git v1.7.8 or newer.
+    # --- >8 --- >8 ---
+    # Longer-term: Use `git merge --no-commit` so that we always
+    # have a chance to explain our merges.
+    os.environ['GIT_MERGE_AUTOEDIT'] = 'no'
+    os.putenv('GIT_MERGE_AUTOEDIT', 'no')
+
 
 @memoize
 def instance(argv):
@@ -126,7 +166,36 @@ class ColaApplication(object):
 
 
 def parse_args(context):
-    parser = optparse.OptionParser(usage='%prog [options]')
+    # TODO switch to argparse, bundle it for win32?
+    args = sys.argv[1:]
+    builtins = set(('archive',
+                    'branch',
+                    'browse',
+                    'classic',
+                    'config',
+                    'dag',
+                    'fetch',
+                    'grep',
+                    'pull',
+                    'push',
+                    'remote',
+                    'stash',
+                    'search',
+                    'tag'))
+    if context in ('git-dag', 'dag'):
+        context = 'dag'
+        usage = 'git dag [options]'
+    elif args and args[0] in builtins:
+        context = args.pop(0)
+        sys.argv = sys.argv[0:1] + args
+        usage = 'git cola %s [options]' % context
+    else:
+        usage = ('git cola [sub-command] [options]\n'
+                 '\n'
+                 'Sub-commands:\n\t' +
+                 '\n\t'.join(sorted(builtins)))
+
+    parser = optparse.OptionParser(usage=usage)
 
     # We also accept 'git cola version'
     parser.add_option('-v', '--version',
@@ -156,14 +225,15 @@ def parse_args(context):
                       metavar='PATH',
                       default='')
 
-    if context == 'git-dag':
+    if context == 'dag':
         parser.add_option('-c', '--count',
                           help='Number of commits to display.',
                           dest='count',
                           type='int',
                           default=1000)
 
-    return parser.parse_args()
+    opts, args = parser.parse_args()
+    return opts, args, context
 
 
 def process_args(opts, args):
@@ -194,7 +264,7 @@ def main(context):
     """Parses the command-line arguments and starts git-cola
     """
     setup_environment()
-    opts, args = parse_args(context)
+    opts, args, context = parse_args(context)
     repo = process_args(opts, args)
 
     # Allow Ctrl-C to exit
@@ -217,44 +287,58 @@ def main(context):
     # Finally, go to the root of the git repo
     os.chdir(model.git.worktree())
 
-    # Prepare to launch a sub-command
-    builtins = set(('cola',
-                    'classic',
-                    'dag',
-                    'fetch',
-                    'pull',
-                    'push',
-                    'stash',
-                    'tag'))
-
-    if context != 'git-dag' and args and args[0] in builtins:
-        context = args[0]
-
     # Show the GUI
-    if context == 'git-cola' or context == 'cola':
-        view = MainView(model, qtutils.active_window())
-        ctl = MainController(model, view)
-    elif context == 'git-dag' or context == 'dag':
+    if context == 'archive':
+        from cola.widgets.archive import GitArchiveDialog
+        model.update_status()
+        view = GitArchiveDialog(model.currentbranch)
+    elif context == 'branch':
+        from cola.widgets.createbranch import create_new_branch
+        view = create_new_branch()
+    elif context in ('git-dag', 'dag'):
+        from cola.dag import git_dag
         ctl = git_dag(model, opts=opts, args=args)
         view = ctl.view
-    elif context == 'classic':
+    elif context in ('classic', 'browse'):
+        from cola.classic import cola_classic
         view = cola_classic(update=False)
-    # TODO: the calls to update_status() can be done asynchronously
-    # by hooking into the message_updated notification.
+    elif context == 'config':
+        from cola.prefs import preferences
+        ctl = preferences()
+        view = ctl.view
+    elif context == 'fetch':
+        # TODO: the calls to update_status() can be done asynchronously
+        # by hooking into the message_updated notification.
+        from cola.widgets import remote
+        model.update_status()
+        view = remote.fetch()
+    elif context == 'grep':
+        from cola.widgets import grep
+        view = grep.run_grep(parent=None)
+    elif context == 'pull':
+        from cola.widgets import remote
+        model.update_status()
+        view = remote.pull()
+    elif context == 'push':
+        from cola.widgets import remote
+        model.update_status()
+        view = remote.push()
+    elif context == 'remote':
+        from cola.widgets import editremotes
+        view = editremotes.edit()
+    elif context == 'search':
+        from cola.widgets.search import search
+        view = search()
     elif context == 'stash':
+        from cola.stash import stash
         model.update_status()
         view = stash().view
-    elif context == 'fetch':
-        model.update_status()
-        view = guicmds.fetch().view
-    elif context == 'pull':
-        model.update_status()
-        view = guicmds.pull().view
-    elif context == 'push':
-        model.update_status()
-        view = guicmds.push().view
     elif context == 'tag':
-        view = create_tag().view
+        from cola.widgets.createtag import create_tag
+        view = create_tag()
+    else:
+        view = MainView(model, qtutils.active_window())
+        ctl = MainController(model, view)
 
     # Install UI wrappers for command objects
     cfgactions.install_command_wrapper()
@@ -282,7 +366,7 @@ def main(context):
     inotify.stop()
     QtCore.QThreadPool.globalInstance().waitForDone()
 
-    pattern = cola.model().tmp_file_pattern()
+    pattern = utils.tmp_file_pattern()
     for filename in glob.glob(pattern):
         os.unlink(filename)
     sys.exit(result)
