@@ -157,6 +157,8 @@ def tracked_branch(branch=None, config=None):
         config = gitcfg.instance()
     if branch is None:
         branch = current_branch()
+    if branch is None:
+        return None
     remote = config.get('branch.%s.remote' % branch)
     if not remote:
         return None
@@ -213,22 +215,12 @@ def sha1_diff(sha1, git=git):
     return core.decode(git.diff(sha1 + '^!', **_common_diff_opts()))
 
 
-def diff_info(sha1, git=git, merge=True):
-
-    if merge:
-        mergefmt = 'Merge: %P%n'
-    else:
-        mergefmt = ''
-
-    log = git.log('-1',
-            '--pretty=format:'
-            'commit %H%n' +
-            mergefmt +
-            'Author: %aN <%aE>%n'
-            'Date:   %aD%n'
-            '%n%s%n%n%b',
-            sha1)
-    return core.decode(log) + '\n\n' + sha1_diff(sha1)
+def diff_info(sha1, git=git):
+    log = git.log('-1', '--pretty=format:%b', sha1)
+    decoded = core.decode(log).strip()
+    if decoded:
+        decoded += '\n\n'
+    return decoded + sha1_diff(sha1)
 
 
 def diff_helper(commit=None,
@@ -236,6 +228,8 @@ def diff_helper(commit=None,
                 endref=None,
                 filename=None,
                 cached=True,
+                head=None,
+                amending=False,
                 with_diff_header=False,
                 suppress_header=True,
                 reverse=False,
@@ -250,19 +244,23 @@ def diff_helper(commit=None,
     elif ref:
         for r in utils.shell_split(ref.strip()):
             argv.append(r)
+    elif head and amending and cached:
+        argv.append(head)
 
+    encoding = None
     if filename:
         argv.append('--')
         if type(filename) is list:
             argv.extend(filename)
         else:
             argv.append(filename)
+            encoding = config.file_encoding(filename)
 
-    start = False
-    del_tag = 'deleted file mode '
 
-    headers = []
-    deleted = cached and not os.path.exists(encode(filename))
+    if filename is not None:
+        deleted = cached and not os.path.exists(encode(filename))
+    else:
+        deleted = False
 
     status, diffoutput = git.diff(R=reverse, M=True, cached=cached,
                                   with_status=True,
@@ -274,15 +272,26 @@ def diff_helper(commit=None,
         else:
             return ''
 
+    return extract_diff_header(status, deleted, encoding,
+                               with_diff_header, suppress_header, diffoutput)
+
+
+def extract_diff_header(status, deleted, encoding,
+                        with_diff_header, suppress_header, diffoutput):
+    encode = core.encode
+    headers = []
+
     if diffoutput.startswith('Submodule'):
         if with_diff_header:
             return ('', diffoutput)
         else:
             return diffoutput
 
+    start = False
+    del_tag = 'deleted file mode '
     output = StringIO()
 
-    diff = core.decode(diffoutput).split('\n')
+    diff = core.decode(diffoutput, encoding=encoding).split('\n')
     for line in diff:
         if not start and '@@' == line[:2] and '@@' in line[2:]:
             start = True
@@ -359,29 +368,31 @@ def export_patchset(start, end, output='patches', **kwargs):
 
 
 def unstage_paths(args, head='HEAD'):
-    status, output = git.reset(head, '--', with_stderr=True, with_status=True,
+    status, output = git.reset(head, '--', with_status=True,
                                *set(args))
-    if status != 128:
+    if status == 128:
+        # handle git init: we have to use 'git rm --cached'
+        # detect this condition by checking if the file is still staged
+        return untrack_paths(args, head=head)
+    else:
         return (status, output)
-    # handle git init: we have to use 'git rm --cached'
-    # detect this condition by checking if the file is still staged
-    status, output = git.update_index('--',
-                                      force_remove=True,
-                                      with_status=True,
-                                      with_stderr=True,
-                                      *set(args))
-    return (status, output)
 
 
+def untrack_paths(args, head='HEAD'):
+    if not args:
+        return (-1, 'Nothing to do')
+    return git.update_index('--', force_remove=True,
+                            with_status=True, *set(args))
 
-def worktree_state(head='HEAD', staged_only=False):
+
+def worktree_state(head='HEAD'):
     """Return a tuple of files in various states of being
 
     Can be staged, unstaged, untracked, unmerged, or changed
     upstream.
 
     """
-    state = worktree_state_dict(head=head, staged_only=staged_only)
+    state = worktree_state_dict(head=head)
     return(state.get('staged', []),
            state.get('modified', []),
            state.get('unmerged', []),
@@ -389,9 +400,7 @@ def worktree_state(head='HEAD', staged_only=False):
            state.get('upstream_changed', []))
 
 
-def worktree_state_dict(head='HEAD',
-                        staged_only=False,
-                        update_index=False):
+def worktree_state_dict(head='HEAD', update_index=False):
     """Return a dict of files in various states of being
 
     :rtype: dict, keys are staged, unstaged, untracked, unmerged,
@@ -401,9 +410,6 @@ def worktree_state_dict(head='HEAD',
     if update_index:
         git.update_index(refresh=True)
 
-    if staged_only:
-        return _branch_status(head)
-
     staged, unmerged, submodules = diff_index(head)
     modified, more_submods = diff_worktree()
 
@@ -435,14 +441,15 @@ def worktree_state_dict(head='HEAD',
             'submodules': submodules}
 
 
-def diff_index(head):
+def diff_index(head, cached=True):
     decode = core.decode
     submodules = set()
     staged = []
     unmerged = []
 
-    output = git.diff_index(head, z=True, cached=True, with_stderr=True)
-    if output.startswith('fatal:'):
+    status, output = git.diff_index(head, cached=cached,
+                                    z=True, with_status=True)
+    if status != 0:
         # handle git init
         return all_files(), unmerged, submodules
 
@@ -465,8 +472,8 @@ def diff_worktree():
     modified = []
     submodules = set()
 
-    output = git.diff_files(z=True, with_stderr=True)
-    if output.startswith('fatal:'):
+    status, output = git.diff_files(z=True, with_status=True)
+    if status != 0:
         # handle git init
         ls_files = core.decode(git.ls_files(modified=True, z=True))
         if ls_files:
@@ -480,7 +487,7 @@ def diff_worktree():
         name = core.decode(name)
         if '160000' in rest[1:14]:
             submodules.add(name)
-        elif status in 'DAM':
+        elif status in 'DAMT':
             modified.append(name)
 
     return modified, submodules
@@ -514,8 +521,8 @@ def merge_base_to(head, ref):
 def merge_base_parent(branch):
     tracked = tracked_branch(branch=branch)
     if tracked:
-        return '%s..%s' % (tracked, branch)
-    return 'master..%s' % branch
+        return tracked
+    return 'HEAD'
 
 
 def eval_path(path):
