@@ -2,7 +2,7 @@
 """Provide cola's version number"""
 
 # The current git-cola version
-_default_version = '1.7.2'
+_default_version = '1.8.0'
 
 
 import os
@@ -66,10 +66,15 @@ def version_to_list(version):
     return ver_list
 
 
+@memoize
+def git_version_str():
+    """Returns the current GIT version"""
+    return git.version()
+
 @memoize
 def git_version():
     """Returns the current GIT version"""
-    return git.version().split()[-1]
+    return git_version_str().split()[-1]
 
 
 if __name__ == '__main__':
