@@ -3,13 +3,16 @@ import subprocess
 import itertools
 
 from PyQt4 import QtGui
+from PyQt4.QtCore import Qt
 from PyQt4.QtCore import SIGNAL
 
 import cola
 from cola import signals
 from cola import qtutils
+from cola import utils
 from cola.compat import set
 from cola.qtutils import SLOT
+from cola.widgets import defs
 from cola.models.selection import State
 
 
@@ -50,6 +53,9 @@ class StatusTreeWidget(QtGui.QTreeWidget):
     idx_untracked = 3
     idx_end = 4
 
+    txt_default_app = 'Open Using Default Application'
+    txt_parent_dir = 'Open Parent Directory'
+
     # Read-only access to the mode state
     mode = property(lambda self: self.m.mode)
 
@@ -77,33 +83,65 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         self.expanded_items = set()
 
         self.process_selection = qtutils.add_action(self,
-                'Process Selection', self._process_selection, 'Ctrl+S')
+                'Process Selection', self._process_selection,
+                defs.stage_shortcut)
 
         self.launch_difftool = qtutils.add_action(self,
-                'Process Selection', self._launch_difftool, 'Ctrl+D')
+                'Launch Diff Tool', self._launch_difftool,
+                defs.difftool_shortcut)
+        self.launch_difftool.setIcon(qtutils.icon('git.svg'))
 
-        self.launch_difftool = qtutils.add_action(self,
-                'Launch Editor', self._launch_editor, 'Ctrl+E')
+        self.launch_editor = qtutils.add_action(self,
+                'Launch Editor', self._launch_editor,
+                defs.editor_shortcut)
+        self.launch_editor.setIcon(qtutils.options_icon())
 
-        self.up = qtutils.add_action(self, 'Move Up', self.move_up, 'K')
-        self.down = qtutils.add_action(self, 'Move Down', self.move_down, 'J')
+        if not utils.is_win32():
+            self.open_using_default_app = qtutils.add_action(self,
+                    self.txt_default_app,
+                    self._open_using_default_app,
+                    defs.default_app_shortcut)
+            self.open_using_default_app.setIcon(qtutils.file_icon())
+
+            self.open_parent_dir = qtutils.add_action(self,
+                    self.txt_parent_dir,
+                    self._open_parent_dir,
+                    defs.parent_dir_shortcut)
+            self.open_parent_dir.setIcon(qtutils.open_file_icon())
+
+        self.up = qtutils.add_action(self,
+                'Move Up', self.move_up, Qt.Key_K)
+        self.down = qtutils.add_action(self,
+                'Move Down', self.move_down, Qt.Key_J)
+
+        self.copy_path_action = qtutils.add_action(self,
+                                                   'Copy Path to Clipboard',
+                                                   self.copy_path,
+                                                   QtGui.QKeySequence.Copy)
+        self.copy_path_action.setIcon(qtutils.theme_icon('edit-copy.svg'))
 
         self.connect(self, SIGNAL('about_to_update'), self._about_to_update)
         self.connect(self, SIGNAL('updated'), self._updated)
 
         self.m = cola.model()
-        self.m.add_message_observer(self.m.message_about_to_update,
-                                    self.about_to_update)
-        self.m.add_message_observer(self.m.message_updated, self.updated)
+        self.m.add_observer(self.m.message_about_to_update,
+                            self.about_to_update)
+        self.m.add_observer(self.m.message_updated, self.updated)
 
         self.connect(self, SIGNAL('itemSelectionChanged()'),
                      self.show_selection)
+
         self.connect(self,
                      SIGNAL('itemDoubleClicked(QTreeWidgetItem*,int)'),
                      self.double_clicked)
+
         self.connect(self,
-                     SIGNAL('itemClicked(QTreeWidgetItem*,int)'),
-                     self.clicked)
+                     SIGNAL('itemCollapsed(QTreeWidgetItem*)'),
+                     lambda x: self.update_column_widths())
+
+        self.connect(self,
+                     SIGNAL('itemExpanded(QTreeWidgetItem*)'),
+                     lambda x: self.update_column_widths())
 
     def add_item(self, txt, path, hide=False):
         """Create a new top-level item in the status tree."""
@@ -229,20 +267,12 @@ class StatusTreeWidget(QtGui.QTreeWidget):
             self.old_scroll = None
 
         self.restore_selection()
-
-        if not self.m.staged:
-            return
-
-        staged = self.topLevelItem(self.idx_staged)
-        if self.mode in self.m.modes_read_only:
-            staged.setText(0, self.tr('Changed'))
-        else:
-            staged.setText(0, self.tr('Staged'))
+        self.update_column_widths()
 
     def set_staged(self, items):
         """Adds items to the 'Staged' subtree."""
         self._set_subtree(items, self.idx_staged, staged=True,
-                          check=not self.m.read_only())
+                          check=not self.m.amending())
 
     def set_modified(self, items):
         """Adds items to the 'Modified' subtree."""
@@ -275,6 +305,9 @@ class StatusTreeWidget(QtGui.QTreeWidget):
             parent.addChild(treeitem)
         self.expand_items(idx, items)
 
+    def update_column_widths(self):
+        self.resizeColumnToContents(0)
+
     def expand_items(self, idx, items):
         """Expand the top-level category "folder" once and only once."""
         # Don't do this if items is empty; this makes it so that we
@@ -312,90 +345,152 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                                    SLOT(signals.unstage_all))
                     return menu
                 elif idx == self.idx_unmerged:
-                    menu.addAction(qtutils.icon('add.svg'),
-                                   self.tr('Stage Merged'),
-                                   SLOT(signals.stage_unmerged))
+                    action = menu.addAction(qtutils.icon('add.svg'),
+                                            self.tr('Stage Merged'),
+                                            SLOT(signals.stage_unmerged))
+                    action.setShortcut(defs.stage_shortcut)
                     return menu
                 elif idx == self.idx_modified:
-                    menu.addAction(qtutils.icon('add.svg'),
-                                   self.tr('Stage Modified'),
-                                   SLOT(signals.stage_modified))
+                    action = menu.addAction(qtutils.icon('add.svg'),
+                                            self.tr('Stage Modified'),
+                                            SLOT(signals.stage_modified))
+                    action.setShortcut(defs.stage_shortcut)
                     return menu
 
                 elif idx == self.idx_untracked:
-                    menu.addAction(qtutils.icon('add.svg'),
-                                   self.tr('Stage Untracked'),
-                                   SLOT(signals.stage_untracked))
+                    action = menu.addAction(qtutils.icon('add.svg'),
+                                            self.tr('Stage Untracked'),
+                                            SLOT(signals.stage_untracked))
+                    action.setShortcut(defs.stage_shortcut)
                     return menu
 
-        enable_staging = self.m.enable_staging()
-        if not enable_staging:
-            menu.addAction(qtutils.icon('remove.svg'),
-                           self.tr('Unstage Selected'),
-                           SLOT(signals.unstage, self.staged()))
+        if s.staged and self.m.unstageable():
+            action = menu.addAction(qtutils.icon('remove.svg'),
+                                    self.tr('Unstage Selected'),
+                                    SLOT(signals.unstage, self.staged()))
+            action.setShortcut(defs.stage_shortcut)
 
         if s.staged and s.staged[0] in self.m.submodules:
             menu.addAction(qtutils.git_icon(),
                            self.tr('Launch git-cola'),
                            SLOT(signals.open_repo,
                                 os.path.abspath(s.staged[0])))
+            menu.addSeparator()
+            menu.addAction(self.copy_path_action)
             return menu
         elif s.staged:
             menu.addSeparator()
-            menu.addAction(qtutils.icon('open.svg'),
-                           self.tr('Launch Editor'),
-                           SLOT(signals.edit, self.staged()))
-            menu.addAction(qtutils.git_icon(),
-                           self.tr('Launch Diff Tool'),
-                           SLOT(signals.difftool, True, self.staged()))
+            action = menu.addAction(qtutils.git_icon(),
+                                    self.tr('Launch Diff Tool'),
+                                    SLOT(signals.difftool, True,
+                                         self.staged()))
+            action.setShortcut(defs.difftool_shortcut)
+
+            action = menu.addAction(qtutils.options_icon(),
+                                    self.tr('Launch Editor'),
+                                    SLOT(signals.edit, self.staged()))
+            action.setShortcut(defs.editor_shortcut)
+
+            if not utils.is_win32():
+                menu.addSeparator()
+                action = menu.addAction(qtutils.file_icon(),
+                        self.tr(self.txt_default_app),
+                        SLOT(signals.open_default_app, self.staged()))
+                action.setShortcut(defs.default_app_shortcut)
+
+                action = menu.addAction(qtutils.open_file_icon(),
+                        self.tr(self.txt_parent_dir),
+                        self._open_parent_dir)
+                action.setShortcut(defs.parent_dir_shortcut)
+
+            if self.m.undoable():
+                menu.addSeparator()
+                menu.addAction(qtutils.icon('undo.svg'),
+                               self.tr('Revert Unstaged Edits...'),
+                               lambda: self._revert_unstaged_edits(staged=True))
             menu.addSeparator()
-            menu.addAction(qtutils.icon('undo.svg'),
-                           self.tr('Revert Unstaged Edits...'),
-                           lambda: self._revert_unstaged_edits(use_staged=True))
+            menu.addAction(self.copy_path_action)
             return menu
 
         if s.unmerged:
             menu.addAction(qtutils.git_icon(),
                            self.tr('Launch Merge Tool'),
                            SLOT(signals.mergetool, self.unmerged()))
-            menu.addAction(qtutils.icon('open.svg'),
-                           self.tr('Launch Editor'),
-                           SLOT(signals.edit, self.unmerged()))
+
+            action = menu.addAction(qtutils.icon('add.svg'),
+                                    self.tr('Stage Selected'),
+                                    SLOT(signals.stage, self.unstaged()))
+            action.setShortcut(defs.stage_shortcut)
             menu.addSeparator()
-            menu.addAction(qtutils.icon('add.svg'),
-                           self.tr('Stage Selected'),
-                           SLOT(signals.stage, self.unstaged()))
+            action = menu.addAction(qtutils.options_icon(),
+                                    self.tr('Launch Editor'),
+                                    SLOT(signals.edit, self.unmerged()))
+            action.setShortcut(defs.editor_shortcut)
+
+            if not utils.is_win32():
+                menu.addSeparator()
+                action = menu.addAction(qtutils.file_icon(),
+                        self.tr(self.txt_default_app),
+                        SLOT(signals.open_default_app, self.unmerged()))
+                action.setShortcut(defs.default_app_shortcut)
+
+                action = menu.addAction(qtutils.open_file_icon(),
+                        self.tr(self.txt_parent_dir),
+                        self._open_parent_dir)
+                action.setShortcut(defs.parent_dir_shortcut)
+
+            menu.addSeparator()
+            menu.addAction(self.copy_path_action)
             return menu
 
         modified_submodule = (s.modified and
                               s.modified[0] in self.m.submodules)
-        if enable_staging:
-            menu.addAction(qtutils.icon('add.svg'),
-                           self.tr('Stage Selected'),
-                           SLOT(signals.stage, self.unstaged()))
+        if self.m.stageable():
+            action = menu.addAction(qtutils.icon('add.svg'),
+                                    self.tr('Stage Selected'),
+                                    SLOT(signals.stage, self.unstaged()))
+            action.setShortcut(defs.stage_shortcut)
             menu.addSeparator()
 
+        if s.modified and self.m.stageable() and not modified_submodule:
+            action = menu.addAction(qtutils.git_icon(),
+                                    self.tr('Launch Diff Tool'),
+                                    SLOT(signals.difftool, False,
+                                         self.modified()))
+            action.setShortcut(defs.difftool_shortcut)
+
         if modified_submodule:
             menu.addAction(qtutils.git_icon(),
                            self.tr('Launch git-cola'),
                            SLOT(signals.open_repo,
                                 os.path.abspath(s.modified[0])))
         elif self.unstaged():
-            menu.addAction(qtutils.icon('open.svg'),
-                           self.tr('Launch Editor'),
-                           SLOT(signals.edit, self.unstaged()))
+            action = menu.addAction(qtutils.options_icon(),
+                                    self.tr('Launch Editor'),
+                                    SLOT(signals.edit, self.unstaged()))
+            action.setShortcut(defs.editor_shortcut)
+            if not utils.is_win32():
+                menu.addSeparator()
+                action = menu.addAction(qtutils.file_icon(),
+                        self.tr(self.txt_default_app),
+                        SLOT(signals.open_default_app, self.unstaged()))
+                action.setShortcut(defs.default_app_shortcut)
 
-        if s.modified and enable_staging and not modified_submodule:
-            menu.addAction(qtutils.git_icon(),
-                           self.tr('Launch Diff Tool'),
-                           SLOT(signals.difftool, False, self.modified()))
-            menu.addSeparator()
-            menu.addAction(qtutils.icon('undo.svg'),
-                           self.tr('Revert Unstaged Edits...'),
-                           self._revert_unstaged_edits)
-            menu.addAction(qtutils.icon('undo.svg'),
-                           self.tr('Revert Uncommited Edits...'),
-                           self._revert_uncommitted_edits)
+                action = menu.addAction(qtutils.open_file_icon(),
+                        self.tr(self.txt_parent_dir),
+                        self._open_parent_dir)
+                action.setShortcut(defs.parent_dir_shortcut)
+
+        menu.addSeparator()
+
+        if s.modified and self.m.stageable() and not modified_submodule:
+            if self.m.undoable():
+                menu.addAction(qtutils.icon('undo.svg'),
+                               self.tr('Revert Unstaged Edits...'),
+                               self._revert_unstaged_edits)
+                menu.addAction(qtutils.icon('undo.svg'),
+                               self.tr('Revert Uncommited Edits...'),
+                               self._revert_uncommitted_edits)
 
         if s.untracked:
             menu.addSeparator()
@@ -406,6 +501,8 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                            self.tr('Add to .gitignore'),
                            SLOT(signals.ignore,
                                 map(lambda x: '/' + x, self.untracked())))
+        menu.addSeparator()
+        menu.addAction(self.copy_path_action)
         return menu
 
     def _delete_files(self):
@@ -426,14 +523,14 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         ok_txt = 'Delete Files'
 
         if qtutils.confirm(title, msg, info_txt, ok_txt,
-                           default=False,
+                           default=True,
                            icon=qtutils.discard_icon()):
             cola.notifier().broadcast(signals.delete, files)
 
-    def _revert_unstaged_edits(self, use_staged=False):
+    def _revert_unstaged_edits(self, staged=False):
         if not self.m.undoable():
             return
-        if use_staged:
+        if staged:
             items_to_undo = self.staged()
         else:
             items_to_undo = self.modified()
@@ -444,18 +541,19 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                                    '\nThese changes cannot be recovered.',
                                    'Revert the unstaged changes?',
                                    'Revert Unstaged Changes',
-                                   default=False,
+                                   default=True,
                                    icon=qtutils.icon('undo.svg')):
                 return
+            args = []
+            if not staged and self.m.amending():
+                args.append(self.m.head)
             cola.notifier().broadcast(signals.checkout,
-                                      ['--'] + items_to_undo)
+                                      args + ['--'] + items_to_undo)
         else:
             qtutils.log(1, self.tr('No files selected for '
                                    'checkout from HEAD.'))
 
     def _revert_uncommitted_edits(self):
-        if not self.m.undoable():
-            return
         items_to_undo = self.modified()
         if items_to_undo:
             if not qtutils.confirm('Revert Uncommitted Changes?',
@@ -463,11 +561,11 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                                    '\nThese changes cannot be recovered.',
                                    'Revert the uncommitted changes?',
                                    'Revert Uncommitted Changes',
-                                   default=False,
+                                   default=True,
                                    icon=qtutils.icon('undo.svg')):
                 return
             cola.notifier().broadcast(signals.checkout,
-                                      ['HEAD', '--'] + items_to_undo)
+                                      [self.m.head, '--'] + items_to_undo)
         else:
             qtutils.log(1, self.tr('No files selected for '
                                    'checkout from HEAD.'))
@@ -517,6 +615,20 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         c = self.contents()
         return c.staged + c.unmerged + c.modified + c.untracked
 
+    def selected_group(self):
+        """A list of selected files in various states of being"""
+        selection = []
+        s = self.selection()
+        if s.staged:
+            selection = s.staged
+        elif s.unmerged:
+            selection = s.unmerged
+        elif s.modified:
+            selection = s.modified
+        elif s.untracked:
+            selection = s.untracked
+        return selection
+
     def selected_idx(self):
         c = self.contents()
         s = self.single_selection()
@@ -583,9 +695,6 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         the a context-specific action.
 
         """
-        if self.m.read_only():
-            return
-
         # Sync the selection model
         s = self.selection()
         cola.selection_model().set_selection(s)
@@ -593,7 +702,7 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         # Clear the selection if an empty area was clicked
         selection = self.selected_indexes()
         if not selection:
-            if self.mode == self.m.mode_amend:
+            if self.m.amending():
                 cola.notifier().broadcast(signals.set_diff_text, '')
             else:
                 cola.notifier().broadcast(signals.reset_mode)
@@ -602,17 +711,11 @@ class StatusTreeWidget(QtGui.QTreeWidget):
             self.blockSignals(False)
             return
 
-        filename = cola.selection_model().filename()
-        if filename is not None:
-            qtutils.set_clipboard(filename)
-
     def double_clicked(self, item, idx):
         """Called when an item is double-clicked in the repo status tree."""
         self._process_selection()
 
     def _process_selection(self):
-        if self.m.read_only():
-            return
         s = self.selection()
         if s.staged:
             cola.notifier().broadcast(signals.unstage, s.staged)
@@ -640,19 +743,24 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         cola.notifier().broadcast(signals.difftool, bool(staged), selection)
 
     def _launch_editor(self):
-        s = self.selection()
-        if s.staged:
-            selection = s.staged
-        elif s.unmerged:
-            selection = s.unmerged
-        elif s.modified:
-            selection = s.modified
-        elif s.untracked:
-            selection = s.untracked
-        else:
+        selection = self.selected_group()
+        if not selection:
             return
         cola.notifier().broadcast(signals.edit, selection)
 
+    def _open_using_default_app(self):
+        selection = self.selected_group()
+        if not selection:
+            return
+        cola.notifier().broadcast(signals.open_default_app, selection)
+
+    def _open_parent_dir(self):
+        selection = self.selected_group()
+        if not selection:
+            return
+        dirs = set(map(os.path.dirname, selection))
+        cola.notifier().broadcast(signals.open_default_app, dirs)
+
     def show_selection(self):
         """Show the selected item."""
         # Sync the selection model
@@ -724,3 +832,10 @@ class StatusTreeWidget(QtGui.QTreeWidget):
             self.select_by_index(idx + 1)
         else:
             self.select_by_index(0)
+
+    def copy_path(self):
+        """Copy a selected path to the clipboard"""
+        filename = cola.selection_model().filename()
+        if filename is not None:
+            curdir = os.getcwdu()
+            qtutils.set_clipboard(os.path.join(curdir, filename))
